<?xml version="1.0" encoding="UTF-8"?>
<book version="5.0" xmlns="http://docbook.org/ns/docbook" xmlns:xi="http://www.w3.org/2001/XInclude">

  <#assign locale = "en" />
  <#assign sysDateTime = .now>

  <#assign left_header_text = ''/> 
	<#assign central_header_text = "Inbound references report" />
	<#assign right_header_text = ''/> 
	
	<#assign left_footer_text = sysDateTime?string["dd/MM/yyyy"] + " - IUCLID 6 " + iuclid6Version!/> 
	<#assign central_footer_text = 'Inbound references report' />
	<#assign right_footer_text = ''/> 

<#import "utils.ftl" as utils>
<#-- Import common macros and functions -->
<#import "macros_common_general.ftl" as com>


<#-- Initialize common variables -->
<@com.initializeMainVariables/>	
<#-- Root entity -->
<#assign rootEntity = entity.root>
<#-- Utility list to remember tarversed entities -->
<#assign traveresedEntityKeyList = [rootEntity.documentKey]>
<#-- Utility list to remember entities yet to be traversed -->
<#assign toTravereseEntityList = []>

 <info>
      <title>Inbound References Report</title>

        <subtitle>
           <para>
                <@com.emptyLine/>
                <#if _subject.documentType=="MIXTURE">Mixture: <@com.text _subject.MixtureName/>
                <#elseif _subject.documentType=="SUBSTANCE">Substance: <@com.text _subject.ChemicalName/></#if>
                <@com.emptyLine/>
          </para>
      </subtitle>


		<cover>
			<para role="cover.rule">
        <guiicon>
              <#if _dossierHeader?has_content>
                <#assign dossier = _dossierHeader />

                <para>
                  <emphasis role="bold">Dossier details:</emphasis>
                    <para role="indent">
                    <itemizedlist>
                        <listitem>Dossier name: <@com.text dossier.name /></listitem>
                        <listitem>Dossier UUID: <@com.text dossier.subjectKey /></listitem>
                        <listitem>Dossier submission remarks: <@com.text dossier.remarks /></listitem>
                        <listitem>Dossier creation date and time: <@com.text dossier.creationDate /></listitem>
                        <listitem>Submission type: <@com.text dossier.submissionType /></listitem>
                        <#if ownerLegalEntity?has_content>
                          <listitem>Submitting legal entity:
                            <@com.text ownerLegalEntity.GeneralInfo.LegalEntityName/>
                          </listitem>
                        </#if>
                    </itemizedlist>
                    </para>
                </para>

              </#if>
        </guiicon>
      </para>
    </cover>
 </info>

 

<part><title>Inbound References</title>

<chapter label="1">
<title role="HEAD-1">Table of Inbound References</title>

  <para>
  This report lists in one view the inbound references for all IUCLID section documents of a dataset or dossier, including its referenced entities.
  </para><@com.emptyLine/>

  <para>
  Inbound references occur when a IUCLID section document has been referenced by another document or entity.
  </para><@com.emptyLine/>

  <para>
  This report shows the following inbound references inside the dataset or dossier which has been generated:
  </para><@com.emptyLine/>
  <para role="indent">
      <itemizedlist>
              <listitem>A IUCLID document referenced by another IUCLID document, such as an Endpoint study record. Column <emphasis>'Is reference a cross reference?'</emphasis> indicates if the reference is a cross reference or not</listitem>
              <listitem>A IUCLID document referenced by another IUCLID entity, such as a complex entity [e.g. Mixture] or a simple entity [e.g. Literature Reference]</listitem>
              <listitem>A IUCLID document's referenced entity which has been referenced by other IUCLID entities or documents</listitem>
      </itemizedlist>
      <@com.emptyLine/>
  </para>
	

<#-- Go throught the found referenced entities and traverse them. NOTE: the list can grow further as entities can reference further entities. Unclear whether this works well in FreeMarker -->
<#-- FreeMarker does not support do-while loops as such do this workaround with maxLoop -->

<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->

<#-- Maxloop sets the limit at which the component hierarchy is travered -->
<#assign maxLoop=5>
   <table border="1">
        <title>Table of Inbound references</title>
          <col width="13%" />
          <col width="15%" />
          <col width="29%" />		
          <col width="29%" />		
          <col width="14%" />			
          <tbody>
            <tr>	                                       
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Source main entity type and name</emphasis></th>  
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Source section name and number</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Source document type and name</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#BFF0D8" ?><emphasis role="bold">Inbound reference document type and name</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#BFF0D8" ?><emphasis role="bold">Is inbound reference a cross reference?</emphasis></th>
            </tr>

            <#-- Traverse the main entity -->
            <@traverseEntity rootEntity />

              <#list 1..maxLoop as x>           
                <#-- Remove documents that have been already traversed -->
                <#assign toTravereseEntityList = utils.removeDocumentWitKey(toTravereseEntityList, traveresedEntityKeyList) >
                <#if !(toTravereseEntityList?has_content)>
                  <#-- Stop looping if there is nothing to traverse -->
                  <#break>
                </#if>
                <#-- Travers each entity from the list -->
                <#list toTravereseEntityList as entityItem>
                  <#assign traveresedEntityKeyList= traveresedEntityKeyList + [entityItem.documentKey]>
                    <@traverseEntity entityItem />
                </#list>          
              </#list>
            </tbody>
      </table>

<#macro traverseEntity entityDoc>
  <#if utils.isCompositeEntityDocument(entityDoc)>
    <@traverseCompositeEntity entityDoc />
  <#else>
    <@traverseSimpleEntity entityDoc />
  </#if>
</#macro>

<#macro traverseSimpleEntity entityDoc>
  <@traverseDoc entityDoc entityDoc 0 />
</#macro>

<#macro traverseCompositeEntity entityDoc>
  <#local tableOfContents=iuclid.localizeTreeFor(entityDoc.documentType, entityDoc.submissionType, entityDoc.documentKey, utils.getInheritedTemplates(entityDoc), utils.getRelatedCategories(entityDoc), utils.getRelatedMixtures(entityDoc))>
  <@traverseToc tableOfContents entityDoc 0 />
</#macro>

<#macro traverseToc sectionNode entityDocument level=0>
  <#if sectionNode?has_content>
    <#-- iterate through the section documents under this section if there are any -->
    <#if sectionNode.content?has_content>
      <#list sectionNode.content as doc>
        <#-- traverse through the document's blocks and fields aka elements -->
        <@traverseDoc doc entityDocument level+1 sectionNode />
      </#list>
    </#if>
    <#-- iterate through the child sections of the node and do a recursive call -->
    <#if sectionNode?children?has_content>
      <#list sectionNode?children as child> 
        <@traverseToc child entityDocument level+1 />    
      </#list>
    </#if>
  </#if>
</#macro>

<#macro traverseDoc docElementNode entityDocument level=0 sectionNode="" isDocument=true>
  <#if docElementNode?has_content>
    <#-- Only document level -->
    <#if isDocument>
      <@printInboundReferencesForDocument docElementNode entityDocument sectionNode />
    </#if>
    <#-- Check if the element node is a reference field (single or multiple) and remember referenced entities for later traversal in toTravereseEntityList -->
    <#assign toTravereseEntityList = utils.checkAndRememberReferencedEntity(docElementNode, toTravereseEntityList, traveresedEntityKeyList) />
    <#-- iterate through the child structure elements of this document element if there are any and do a recursive call -->
    <#if docElementNode?children?has_content>
      <#list docElementNode?children as child>
        <@traverseDoc child entityDocument level+1 sectionNode false/>
      </#list>
    </#if>
  </#if>
</#macro>

<#macro printInboundReferencesForDocument sectionDoc entityDocument sectionNode>

  <#assign sectionNameForSourceDoc><@sectionNodeDetails sectionNode/></#assign>

  

  <#-- Section doc type -->
  <#local sectionDocType = sectionDoc.documentType>

  <#-- Section sub doc type -->
  <#local sectionDocSubType = sectionDoc.documentSubType>

  <#if !utils.isEntityDocument(sectionDoc)>
    <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
  </#if>

  <#-- The web URL pointing to the document -->
  <#local docUrl=utils.getDocumentUrl(sectionDoc) />

   <#list com.inboundReferences(sectionDoc.documentKey) as inboundReferenceKey>
        
        <#if inboundReferenceKey?has_content>
        <#assign inboundReference = iuclid.getDocumentForKey(inboundReferenceKey) />
            <#if inboundReference?has_content>
            
                <#local docUrl=utils.getDocumentUrl(sectionDoc) />
                <#local docUrlInboundReference=utils.getDocumentUrl(inboundReference) />
                <@iuclid.label for=inboundReference var="referenceFieldLabel"/>	

                    <@tableOutput inboundReference referenceFieldLabel entityDocument sectionNode sectionNameForSourceDoc sectionDoc docUrl docUrlInboundReference />
            </#if>
        </#if>
    </#list>
  

</#macro>

<#macro sectionNodeDetails sectionNode>
  <#if sectionNode?has_content>
    <para><@com.text sectionNode.number /> <@com.text sectionNode.title /></para>
  </#if>
</#macro>

<#macro tableOutput reference referenceFieldLabel entityDocument sectionNode sectionNameForInboundReference sectionDoc docUrl docUrlInboundReference>
  <tr>

     <#-- IUCLID source entity type and name  -->
    <td>
      <@com.emptyLine/>

      <@com.text entityDocument.name/>
        
        <@com.emptyLine/>

        ${entityDocument.documentType}
      
    </td>   

    <#-- IUCLID source Section name and number of -->
    <td>
      <@com.emptyLine/>
    ${sectionNameForInboundReference} 
    </td> 
    
    <#-- IUCLID source document type and name -->
    <td>
      <@com.emptyLine/>

      Entity Type: <emphasis role="bold"><@com.text sectionDoc.documentType/></emphasis>

      <@com.emptyLine/>

      <#if sectionDoc?has_content>
        <ulink url="${docUrl}"><@com.text sectionDoc.name /></ulink>      
      </#if>  
    </td>

    <#-- Inbound reference name and type -->				
    <td>
      <@com.emptyLine/>

      Document Name: <emphasis role="bold"><@com.text referenceFieldLabel /></emphasis>
      <@com.emptyLine/>
      Entity Type: <emphasis role="bold"><@com.text reference.documentType/></emphasis>
      <@com.emptyLine/>
        <ulink url="${docUrlInboundReference}">
          <@com.text reference.name />
        </ulink> 
    </td>	
      
    <#-- Is inbound reference cross reference  -->
    <td>
    <#if isCrossReference(reference, sectionDoc)>
					Yes: Cross-reference made from an Endpoint Study Record						
						<#else>
						No
					</#if>
    </td>	
  </tr>
</#macro>

<#function isCrossReference inboundReference sectionDoc>

<#if inboundReference.documentType=="ENDPOINT_STUDY_RECORD">
      <#return true>
        <#else>
        <#return false>
</#if>
<#return false>
</#function>

<#function getReferencedEntitiesFromDoc docElementNode>
    <#local listOfReferences = [] />

    <#if docElementNode?node_type == "document_reference">
      <#local listOfReferences = checkAndAddReference(docElementNode, listOfReferences) />
      <#return listOfReferences />
    </#if>

    <#if docElementNode?node_type == "document_references">
      <#list docElementNode as item>
        <#local listOfReferences = checkAndAddReference(item, listOfReferences) />
      </#list>
      <#return listOfReferences />
    </#if>
    
    <#if docElementNode?children?has_content>   
      <#list docElementNode?children as child>
      <#local listOfReferences = listOfReferences + getReferencedEntitiesFromDoc(child)  />
      </#list>
    </#if>
    <#return listOfReferences/>
</#function>

<#function checkAndAddReference docKey listOfAllReferences>
  <#local doc = iuclid.getDocumentForKey(docKey) />
  <#if doc?has_content>
    <#if doc.documentType == "CONTACT" || doc.documentType == "TEST_MATERIAL_INFORMATION" || doc.documentType == "SITE" || doc.documentType == "LITERATURE" || doc.documentType == "LEGAL_ENTITY" || doc.documentType == "REFERENCE_SUBSTANCE" || doc.documentType == "ENDPOINT_STUDY_RECORD" || doc.documentType == "ENDPOINT_SUMMARY" || doc.documentType == "FIXED_RECORD" || doc.documentType == "FLEXIBLE_SUMMARY" || doc.documentType == "FLEXIBLE_RECORD" || doc.documentType == "CATEGORY">
        <#return utils.addDocumentToSequenceAsUnique(doc, listOfAllReferences)>
    </#if>
  </#if>
  <#return listOfAllReferences>
</#function>

</chapter>

</part>
</book>
